#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Sample program showing how to add, modify and delete Exif metadata.

import locale
import sys

import exiv2

def main():
    locale.setlocale(locale.LC_ALL, '')
    try:
        exiv2.XmpParser.initialize()
        if len(sys.argv) != 2:
            print('Usage: {} file'.format(sys.argv[0]))
            return 1;
        file = sys.argv[1]
        # Container for exif metadata. This is an example of creating
        # exif metadata from scratch. If you want to add, modify, delete
        # metadata that exists in an image, start with
        # ImageFactory::open
        exifData = exiv2.ExifData()

        # **************************************************************
        # Add to the Exif data

        # This is the quickest way to add (simple) Exif data. If a
        # metadatum for a given key already exists, its value is
        # overwritten. Otherwise a new tag is added.
        exifData["Exif.Image.Model"] = "Test 1"                 # AsciiValue
        exifData["Exif.Image.SamplesPerPixel"] = exiv2.UShortValue(162) # UShortValue
        exifData["Exif.Image.XResolution"] = exiv2.LongValue(-2)        # LongValue
        exifData["Exif.Image.YResolution"] = exiv2.RationalValue((-2, 3)) # Rational
        print("Added a few tags the quick way.")

        # Create a ASCII string value (note the use of create)
        v = exiv2.Value.create(exiv2.TypeId.asciiString)
        # Set the value to a string
        v.read("1999:12:31 23:59:59")
        # Add the value together with its key to the Exif data container
        key = exiv2.ExifKey("Exif.Photo.DateTimeOriginal")
        exifData.add(key, v)
        print('Added key "{}", value "{}"'.format(key, v))

        # Now create a more interesting value (without using the create method)
        rv = exiv2.URationalValue()
        # Set two rational components from a string
        rv.read("1/2 1/3")
        # Add more elements directly
        rv.append((2, 3))
        rv.append((3, 4))
        # Add the key and value pair to the Exif data
        key = exiv2.ExifKey("Exif.Image.PrimaryChromaticities")
        exifData.add(key, rv)
        print('Added key "{}", value "{}"'.format(key, rv))

        # **************************************************************
        # Modify Exif data

        # Since we know that the metadatum exists (or we don't mind
        # creating a new tag if it doesn't), we can simply do this:
        tag = exifData["Exif.Photo.DateTimeOriginal"]
        date = tag.toString()
        date = '2000' + date[4:]
        tag.setValue(date)
        print('Modified key "{}", new value "{}"'.format(
            tag.key(), tag.value()))

        # Alternatively, we can use findKey()
        key = exiv2.ExifKey("Exif.Image.PrimaryChromaticities")
        pos = exifData.findKey(key)
        if pos == exifData.end():
            raise exiv2.AnyError("Key not found")
        # Get a copy of the value
        v = pos.getValue()
        # Downcast the Value pointer to its actual type
        rv = exiv2.URationalValue(v)
        # Modify the value directly
        rv[2] = 88, 77

        # **************************************************************
        # Delete metadata from the Exif data container

        # Delete the metadatum at iterator position pos
        key = exiv2.ExifKey("Exif.Image.PrimaryChromaticities")
        pos = exifData.findKey(key)
        if pos == exifData.end():
            raise exiv2.AnyError("Key not found")
        exifData.erase(pos)
        print('Deleted key "{}"'.format(key))

        # **************************************************************
        # Finally, write the remaining Exif data to the image file
        image = exiv2.ImageFactory.open(file)

        image.setExifData(exifData)
        image.writeMetadata()

        return 0
    except exiv2.AnyError as e:
        print('Caught Exiv2 exception "{}"'.format(str(e)))
        return -1;
    finally:
        exiv2.XmpParser.terminate()
    return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Sample/test for high level XMP classes. See also addmoddel.py.

import locale
import sys

import exiv2

def main():
    locale.setlocale(locale.LC_ALL, '')
    try:
        exiv2.XmpParser.initialize()

        # The XMP property container
        xmpData = exiv2.XmpData()

        # --------------------------------------------------------------
        # Teaser: Setting XMP properties doesn't get much easier than
        # this:

        xmpData["Xmp.dc.source"]  = "xmpsample.cpp"     # a simple text value
        xmpData["Xmp.dc.subject"] = "Palmtree"          # an array item
        xmpData["Xmp.dc.subject"] = "Rubbertree"        # add a 2nd array item
        # a language alternative with two entries and without default
        xmpData["Xmp.dc.title"]   = "lang=de-DE Sonnenuntergang am Strand"
        xmpData["Xmp.dc.title"]   = "lang=en-US Sunset on the beach"

        # --------------------------------------------------------------
        # Any properties can be set provided the namespace is known.
        # Values of any type can be assigned to an Xmpdatum, if they
        # have an output operator. The default XMP value type for
        # unknown properties is a simple text value.

        xmpData["Xmp.dc.one"]     = -1
        xmpData["Xmp.dc.two"]     = 3.1415
        xmpData["Xmp.dc.three"]   = exiv2.RationalValue((5, 7))
        xmpData["Xmp.dc.four"]    = exiv2.UShortValue(255)
        xmpData["Xmp.dc.five"]    = 256
        xmpData["Xmp.dc.six"]     = False
 
    except exiv2.AnyError as e:
        print('Caught Exiv2 exception "{}"'.format(str(e)))
        return -1;
    finally:
        exiv2.XmpParser.terminate()
    return 0

if __name__ == "__main__":
    sys.exit(main())




import logging
import os
import sys
import warnings

if sys.platform == 'win32':
    _dir = os.path.join(os.path.dirname(__file__), 'lib')
    if os.path.isdir(_dir):
        if hasattr(os, 'add_dll_directory'):
            os.add_dll_directory(_dir)
        os.environ['PATH'] = _dir + ';' + os.environ['PATH']

_logger = logging.getLogger(__name__)

class Exiv2Error(Exception):
    """Python exception raised by exiv2 library errors"""
    pass

if sys.version_info < (3, 7):
    # provide old AnyError for compatibility
    AnyError = Exiv2Error
else:
    # issue deprecation warning if user imports AnyError
    def __getattr__(name):
        if name == 'AnyError':
            warnings.warn("Please replace 'AnyError' with 'Exiv2Error'",
                          DeprecationWarning)
            return Exiv2Error
        raise AttributeError

__version__ = "0.14.1"

from exiv2.basicio import *
from exiv2.datasets import *
from exiv2.easyaccess import *
from exiv2.error import *
from exiv2.exif import *
from exiv2.image import *
from exiv2.iptc import *
from exiv2.metadatum import *
from exiv2.preview import *
from exiv2.properties import *
from exiv2.tags import *
from exiv2.types import *
from exiv2.value import *
from exiv2.version import *
from exiv2.xmp import *

_dir = os.path.join(os.path.dirname(__file__), 'messages')
if os.path.isdir(_dir):
    exiv2.types._set_locale_dir(_dir)

__all__ = [x for x in dir() if x[0] != '_']



#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Sample program showing how to set the Exif comment of an image.

import locale
import sys

import exiv2

def main():
    locale.setlocale(locale.LC_ALL, '')
    try:
        exiv2.XmpParser.initialize()
        if len(sys.argv) != 2:
            print('Usage: {} file'.format(sys.argv[0]))
            return 1;
        file = sys.argv[1]

        image = exiv2.ImageFactory.open(file)
        image.readMetadata()
        exifData = image.exifData()

        # Exiv2 uses a CommentValue for Exif user comments. The format
        # of the comment string includes an optional charset
        # specification at the beginning:
 
        # [charset=[Ascii|Jis|Unicode|Undefined]] comment
 
        # Undefined is used as a default if the comment doesn't start
        # with a charset definition.
 
        # Following are a few examples of valid comments. The last one
        # is written to the file.

        exifData["Exif.Photo.UserComment"] = (
            "charset=Unicode A Unicode Exif comment added with Exiv2")
        exifData["Exif.Photo.UserComment"] = (
            "charset=Undefined An undefined Exif comment added with Exiv2")
        exifData["Exif.Photo.UserComment"] = (
            "Another undefined Exif comment added with Exiv2")
        exifData["Exif.Photo.UserComment"] = (
            "charset=Ascii An ASCII Exif comment added with Exiv2")

        print("Writing user comment '{}' back to the image.".format(
            exifData["Exif.Photo.UserComment"]))
 
        image.writeMetadata()

        return 0
    except exiv2.AnyError as e:
        print('Caught Exiv2 exception "{}"'.format(str(e)))
        return -1;
    finally:
        exiv2.XmpParser.terminate()
    return 0

if __name__ == "__main__":
    sys.exit(main())



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2022  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import unittest

import exiv2


class TestMiscellaneous(unittest.TestCase):
    def test_xmp_properties(self):
        ns = exiv2.XmpProperties.registeredNamespaces()
        self.assertIsInstance(ns, dict)
        self.assertEqual(ns['exif'], 'http://ns.adobe.com/exif/1.0/')
        info = exiv2.XmpProperties.nsInfo('exif')
        self.assertEqual(info.ns_, 'http://ns.adobe.com/exif/1.0/')
        self.assertEqual(info.prefix_, 'exif')
        self.assertEqual(info.desc_, 'Exif schema for Exif-specific Properties')
        props = info.xmpPropertyInfo_
        self.assertEqual(props[0].name_, 'ExifVersion')
        props = exiv2.XmpProperties.propertyList('exif')
        self.assertEqual(props[0].name_, 'ExifVersion')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Accessing some Exiv2::Value types directly, without going via string
# representations. For most purposes, using string representations of
# values is good enough, but for maximum speed and precision you can use
# more appropriate Python types.

import datetime
from fractions import Fraction
import locale
import sys

import exiv2
import tzlocal

def main():
    locale.setlocale(locale.LC_ALL, '')
    print('==== DateValue & TimeValue ====')
    # These are only used in Iptc - Exif & Xmp use string representations
    tz = tzlocal.get_localzone()
    py_datetime = datetime.datetime.now(tz)
    print("Python datetime:", py_datetime)
    # Python -> Exiv2
    # can pass ints to constructor or use setDate() afterwards
    date = exiv2.DateValue(py_datetime.year, py_datetime.month, py_datetime.day)
    print("Exiv2 date:", date)
    time = exiv2.TimeValue()
    # negative offsets with non-zero minutes will not be handled
    # correctly by exiv2
    offset = int(tz.utcoffset(py_datetime).total_seconds()) // 60
    time.setTime(py_datetime.hour, py_datetime.minute, py_datetime.second,
                 offset // 60, offset % 60)
    print("Exiv2 time:", time)
    # Exiv2 -> Python
    d = date.getDate()
    t = time.getTime()
    tz = datetime.timezone(
        datetime.timedelta(hours=t.tzHour, minutes=t.tzMinute))
    py_datetime = datetime.datetime(
        d.year, d.month, d.day, t.hour, t.minute, t.second, tzinfo=tz)
    print("Python datetime:", py_datetime)

    print('==== ShortValue ====')
    # This stores 1 or more 16-bit ints.
    py_shorts = [34, 56, 78]
    print("Python short:", py_shorts)
    # Python -> Exiv2, the long way
    shorts = exiv2.ShortValue()
    # append values to initialise
    for x in py_shorts:
        shorts.append(x)
    print("Exiv2 short:", shorts)
    # Python -> Exiv2, the short way
    shorts = exiv2.ShortValue(py_shorts)
    print("Exiv2 short:", shorts)
    # modify a value by index
    shorts[1] = 12
    print("Exiv2 short:", shorts)
    # append a value
    shorts.append(90)
    print("Exiv2 short:", shorts)
    # Exiv2 -> Python
    py_shorts = list(shorts)
    print("Python short:", py_shorts)

    print('==== RationalValue ====')
    # This stores 1 or more pairs of ints. Typical usage is
    # Exif.GPSInfo.GPSLatitude, which stores degrees, minutes & seconds.
    py_latitude = [Fraction(51), Fraction(30),
                   Fraction(4.6).limit_denominator(100000)]
    print("Python rational:", py_latitude)
    # Python -> Exiv2, the long way
    latitude = exiv2.RationalValue()
    # append values to initialise
    for x in py_latitude:
        latitude.append((x.numerator, x.denominator))
    print("Exiv2 rational:", latitude)
    # Python -> Exiv2, the short way
    latitude = exiv2.RationalValue(
        [(x.numerator, x.denominator) for x in py_latitude])
    print("Exiv2 rational:", latitude)
    # modify a value by index
    latitude[1] = -63, 11
    print("Exiv2 rational:", latitude)
    # append a value
    latitude.append((19, 3))
    print("Exiv2 rational:", latitude)
    # Exiv2 -> Python
    py_latitude = [Fraction(*x) for x in latitude]
    print("Python rational:", py_latitude)

    print('==== XmpArrayValue ====')
    # This is used for XmpSeq, XmpBag and XmpAlt values. It stores one
    # or more strings.
    py_seq = ['First string', 'Second', 'Third']
    print("Python seq:", py_seq)
    # Python -> Exiv2, the long way
    seq = exiv2.XmpArrayValue(exiv2.TypeId.xmpBag)
    # append values to initialise
    for x in py_seq:
        seq.append(x)
    print("Exiv2 seq:", seq)
    # Python -> Exiv2, the short way
    seq = exiv2.XmpArrayValue(py_seq, exiv2.TypeId.xmpBag)
    print("Exiv2 seq:", seq)
    # Exiv2 -> Python
    py_seq = list(seq)
    print("Python seq:", py_seq)

    print('==== LangAltValue ====')
    # Used to store text with default language and other language alternatives
    py_langalt = {'x-default': 'default', 'de': 'Deutsch', 'fr': 'French'}
    print("Python langalt:", py_langalt)
    # Python -> Exiv2, the long way
    langalt = exiv2.LangAltValue()
    for key in py_langalt:
        langalt[key] = py_langalt[key]
    print("Exiv2 langalt:", langalt)
    # Python -> Exiv2, the short way
    langalt = exiv2.LangAltValue(py_langalt)
    print("Exiv2 langalt:", langalt)
    print('keys', langalt.keys())
    print('values', langalt.values())
    print('items', langalt.items())
    # delete a value
    del langalt['fr']
    # add a value
    langalt['nl'] = 'Nederlands'
    # Exiv2 -> Python
    py_langalt = dict(langalt)
    print("Python langalt:", py_langalt)

    print('==== DataValue ====')
    py_data = b'0123456789'
    print("Python data:", py_data)
    # Python -> Exiv2
    data = exiv2.DataValue(py_data)
    print("Exiv2 data:", data)
    # Exiv2 -> Python
    py_data = bytearray(len(data))
    data.copy(py_data)
    print("Python data:", py_data)

    return 0


if __name__ == "__main__":
    sys.exit(main())



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2022  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import random
import unittest

import exiv2


class TestBuffers(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        exiv2.XmpParser.initialize()
        test_dir = os.path.dirname(__file__)
        cls.path = os.path.join(test_dir, 'image_02.jpg')

    def test_Image_io(self):
        image = exiv2.ImageFactory.open(self.path)
        io = image.io()
        self.assertEqual(io.open(), 0)
        with memoryview(io.mmap()) as image_data:
            with open(self.path, 'rb') as in_file:
                self.assertEqual(image_data, in_file.read())
        self.assertEqual(io.munmap(), 0)
        self.assertEqual(io.close(), 0)

    def test_DataValue(self):
        py_data_1 = bytes(random.choices(range(256), k=128))
        exv_data = exiv2.DataValue(py_data_1)
        py_data_2 = bytearray(len(exv_data))
        exv_data.copy(py_data_2)
        self.assertEqual(py_data_1, py_data_2)

    def test_Thumb(self):
        image = exiv2.ImageFactory.open(self.path)
        image.readMetadata()
        thumb = exiv2.ExifThumb(image.exifData()).copy()
        self.assertEqual(
            memoryview(thumb.data())[:10], b'\xff\xd8\xff\xe0\x00\x10JFIF')


if __name__ == '__main__':
    unittest.main()



# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.1.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
# Pull in all the attributes from the low-level C/C++ module
if __package__ or "." in __name__:
    from ._types import *
else:
    from _types import *


import enum
AccessMode = enum.IntEnum('AccessMode', _enum_list_AccessMode())
AccessMode.__doc__ = "An identifier for each mode of metadata support."


import enum
ByteOrder = enum.IntEnum('ByteOrder', _enum_list_ByteOrder())
ByteOrder.__doc__ = "Type to express the byte order (little or big endian)."


import enum
MetadataId = enum.IntEnum('MetadataId', _enum_list_MetadataId())
MetadataId.__doc__ = "An identifier for each type of metadata."


import enum
TypeId = enum.IntEnum('TypeId', _enum_list_TypeId())
TypeId.__doc__ = "Exiv2 value type identifiers.\n"
"\nUsed primarily as identifiers when creating Exiv2 Value instances. See"
"\nexiv2.Value.create(). 0x0000 to 0xffff are reserved for TIFF (Exif) types."





#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Sample program to format exif data in various external formats.

import locale
import sys

import exiv2


def syntax(argv, formats):
    print("Usage: {} file format".format(argv[0]))
    print("formats: {}".format(' | '.join(formats)))


def formatJSON(exifData):
    count = 0
    length = exifData.count()
    print('{')
    for datum in exifData:
        count += 1
        print('  "{}":"{}"{}'.format(
            datum.key(), str(datum.value()).replace('"', r'\"'),
            ('',',')[count != length]))
    print('}')


def formatXML(exifData):
    count = 0
    length = exifData.count()
    print('<exif>')
    for datum in exifData:
        count += 1
        print('  <{key}>{value}<{key}/>'.format(
            key=datum.key(),
            value=str(datum.value()).replace('<', '&lt;').replace('>', '&gt')))
    print('</exif>')


def main():
    locale.setlocale(locale.LC_ALL, '')
    try:
        exiv2.XmpParser.initialize()

        formats = {
            "json": formatJSON,
            "xml": formatXML,
            }

        if len(sys.argv) != 3:
            syntax(sys.argv, formats)
            return 1;
        file = sys.argv[1]
        format_ = sys.argv[2]

        if format_ not in formats:
            print("Unrecognised format {}".format(format_))
            syntax(sys.argv, formats)
            return 2;

        image = exiv2.ImageFactory.open(file)
        image.readMetadata()
        exifData = image.exifData()

        formats[format_](exifData)

        return 0
    except exiv2.AnyError as e:
        print('*** error exiv2 exception "{}" ***'.format(str(e)))
        return 4;
    except Exception as e:
        print('*** error exception "{}" ***'.format(str(e)))
        return 5;
    finally:
        exiv2.XmpParser.terminate()
    return 0

if __name__ == "__main__":
    sys.exit(main())



# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.1.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
import exiv2.types

# Pull in all the attributes from the low-level C/C++ module
if __package__ or "." in __name__:
    from ._value import *
else:
    from _value import *


import enum
CharsetId = enum.IntEnum('CharsetId', _enum_list_CharsetId())
CharsetId.__doc__ = "Character set identifiers for the character sets defined by Exif."


import enum
XmpArrayType = enum.IntEnum('XmpArrayType', _enum_list_XmpArrayType())
XmpArrayType.__doc__ = "XMP array types."


import enum
XmpStruct = enum.IntEnum('XmpStruct', _enum_list_XmpStruct())
XmpStruct.__doc__ = "XMP structure indicator."





##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2022  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import sys
import unittest

import exiv2


class TestReferenceCounts(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        exiv2.XmpParser.initialize()
        test_dir = os.path.dirname(__file__)
        cls.image_path = os.path.join(test_dir, 'image_02.jpg')
        cls.image = exiv2.ImageFactory.open(cls.image_path)
        cls.image.readMetadata()

    def test_data(self):
        self.assertEqual(sys.getrefcount(self.image), 2)
        exifData = self.image.exifData()
        self.assertEqual(sys.getrefcount(exifData), 2)
        datum = exifData['Exif.Image.ImageDescription']
        self.assertEqual(sys.getrefcount(datum), 2)
        value = datum.value()
        self.assertEqual(sys.getrefcount(datum), 3)
        del value
        self.assertEqual(sys.getrefcount(datum), 2)
        self.assertEqual(sys.getrefcount(exifData), 3)
        del datum
        self.assertEqual(sys.getrefcount(exifData), 2)
        self.assertEqual(sys.getrefcount(self.image), 3)
        del exifData
        self.assertEqual(sys.getrefcount(self.image), 2)

    def test_memio(self):
        with open(self.image_path, 'rb') as f:
            buf = f.read()
        self.assertEqual(sys.getrefcount(buf), 2)
        image = exiv2.ImageFactory.open(buf)
        self.assertEqual(sys.getrefcount(buf), 3)
        del image
        self.assertEqual(sys.getrefcount(buf), 2)

    def test_io(self):
        self.assertEqual(sys.getrefcount(self.image), 2)
        io = self.image.io()
        self.assertEqual(sys.getrefcount(io), 2)
        io.open()
        mmap = io.mmap()
        self.assertEqual(sys.getrefcount(io), 3)
        del mmap
        self.assertEqual(sys.getrefcount(io), 2)
        self.assertEqual(sys.getrefcount(self.image), 3)
        del io
        self.assertEqual(sys.getrefcount(self.image), 2)

    def test_iterator(self):
        exifData = self.image.exifData()
        self.assertEqual(sys.getrefcount(exifData), 2)
        # iterators point into exifData, so keep a reference to it
        iterator = exifData.begin()
        self.assertEqual(sys.getrefcount(exifData), 3)
        # creating a new iterator increments the parent reference count
        self.assertEqual(sys.getrefcount(iterator), 2)
        new_iterator = iter(iterator)
        self.assertEqual(sys.getrefcount(iterator), 3)
        del new_iterator
        self.assertEqual(sys.getrefcount(iterator), 2)
        del iterator
        self.assertEqual(sys.getrefcount(exifData), 2)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# The quickest way to access, set or modify IPTC metadata.

import locale
import sys

import exiv2

def main():
    locale.setlocale(locale.LC_ALL, '')
    try:
        exiv2.XmpParser.initialize()
        if len(sys.argv) != 2:
            print('Usage: {} file'.format(sys.argv[0]))
            return 1;
        file = sys.argv[1]

        iptcData = exiv2.IptcData()

        iptcData["Iptc.Application2.Headline"] = "The headline I am"
        iptcData["Iptc.Application2.Keywords"] = "Yet another keyword"
        iptcData["Iptc.Application2.DateCreated"] = "2004-08-03"
        iptcData["Iptc.Application2.Urgency"] = exiv2.UShortValue(1)
        iptcData["Iptc.Envelope.ModelVersion"] = 42
        iptcData["Iptc.Envelope.TimeSent"] = "14:41:00-05:00"
        iptcData["Iptc.Application2.RasterizedCaption"] = "230 42 34 2 90 84 23 146"
        iptcData["Iptc.0x0009.0x0001"] = "Who am I?"

        value = exiv2.StringValue()
        value.read("very!")
        iptcData["Iptc.Application2.Urgency"] = value

        print("Time sent: {}".format(iptcData["Iptc.Envelope.TimeSent"]))

        # Open image file
        image = exiv2.ImageFactory.open(file)

        # Set IPTC data and write it to the file
        image.setIptcData(iptcData)
        image.writeMetadata()

        return 0
    except exiv2.AnyError as e:
        print('Caught Exiv2 exception "{}"'.format(str(e)))
        return -1;
    finally:
        exiv2.XmpParser.terminate()
    return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Sample program to print value of an exif key in an image.

import locale
import sys

import exiv2


def main():
    locale.setlocale(locale.LC_ALL, '')
    try:
        exiv2.XmpParser.initialize()

        if len(sys.argv) != 3:
            print("Usage: {} file key".format(sys.argv[0]))
            return 1

        file = sys.argv[1]
        key = sys.argv[2]

        image = exiv2.ImageFactory.open(file)
        image.readMetadata()
        exifData = image.exifData()
        if exifData.empty():
            print("No metadata found in file", file)
            return 2

        print(exifData[key])

        return 0
    except exiv2.AnyError as e:
        print('*** error exiv2 exception "{}" ***'.format(str(e)))
        return 4;
    except Exception as e:
        print('*** error exception "{}" ***'.format(str(e)))
        return 5;
    finally:
        exiv2.XmpParser.terminate()
    return 0

if __name__ == "__main__":
    sys.exit(main())



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2022  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import unittest

import exiv2


class TestPreviewManager(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        exiv2.XmpParser.initialize()
        test_dir = os.path.dirname(__file__)
        cls.path = os.path.join(test_dir, 'image_02.jpg')

    def test_manager(self):
        image = exiv2.ImageFactory.open(self.path)
        image.readMetadata()
        manager = exiv2.PreviewManager(image)
        del image
        props = manager.getPreviewProperties()
        prop = props[0]
        del props
        self.assertEqual(prop.width_, 160)
        preview = manager.getPreviewImage(prop)
        del manager, prop
        self.assertEqual(preview.width(), 160)
        self.assertEqual(
            bytes(preview.pData())[:10], b'\xff\xd8\xff\xe0\x00\x10JFIF')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Sample program to print Exif data from an image.

import locale
import sys

import exiv2


def main():
    locale.setlocale(locale.LC_ALL, '')
    try:
        exiv2.XmpParser.initialize()

        if len(sys.argv) != 2:
            print("Usage: {} [ path | --version | --version-test ]".format(
                sys.argv[0]))
            return 1;
        file = sys.argv[1]

        if file == '--version':
            # Python interface won't build dumpLibraryInfo
##            Exiv2::dumpLibraryInfo(std::cout, keys)
            print(exiv2.versionString())
            return 0;

        if file == '--version-test':
            # verifies/test macro EXIV2_TEST_VERSION
            # described in include/exiv2/version.hpp
            # Python interface doesn't include C++ macros!
##            print("EXV_PACKAGE_VERSION             ", EXV_PACKAGE_VERSION)
            print("Exiv2::version()                ", exiv2.version())
            print("strlen(Exiv2::version())        ", len(exiv2.version()))
            print("Exiv2::versionNumber()          ", exiv2.versionNumber())
            print("Exiv2::versionString()          ", exiv2.versionString())
            print("Exiv2::versionNumberHexString() ",
                  exiv2.versionNumberHexString())
     
            # Test the Exiv2 version available at runtime but compile
            # the if-clause only if the compile-time version is at least
            # 0.15. Earlier versions didn't have a testVersion()
            # function:
            if hasattr(exiv2, 'testVersion'):
                if exiv2.testVersion(0,13,0):
                    print("Available Exiv2 version is equal to or greater"
                          " than 0.13")
                else:
                    print("Installed Exiv2 version is less than 0.13")
            else:
                  print("Compile-time Exiv2 version doesn't have"
                        " exiv2.testVersion()")
            return 0

        image = exiv2.ImageFactory.open(file)
        image.readMetadata()
        exifData = image.exifData()
        if exifData.empty():
            raise exiv2.AnyError("No Exif data found in file")

        end = exifData.end()
        i = exifData.begin()
        while i != end:
            print('{:44s} {:04x} {:9s} {:3d} {:s}'.format(
                i.key(), i.tag(), i.typeName(), i.count(), i.toString()))
            next(i)

        return 0
    except exiv2.AnyError as e:
        print('*** error exiv2 exception "{}" ***'.format(str(e)))
        return 4;
    except Exception as e:
        print('*** error exception "{}" ***'.format(str(e)))
        return 5;
    finally:
        exiv2.XmpParser.terminate()
    return 0

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Sample program to show use of "EasyAccess API".
# See https://github.com/Exiv2/exiv2/wiki/EasyAccess-API

import locale
import sys

import exiv2


def main():
    locale.setlocale(locale.LC_ALL, '')
    exiv2.XmpParser.initialize()

    if len(sys.argv) != 2:
        print("Usage: {} file".format(sys.argv[0]))
        return 1

    file = sys.argv[1]

    image = exiv2.ImageFactory.open(file)
    image.readMetadata()
    exifData = image.exifData()

    for name in ('make', 'model', 'dateTimeOriginal', 'exposureTime',
                 'apertureValue', 'exposureBiasValue', 'exposureIndex', 'flash',
                 'flashBias', 'flashEnergy', 'focalLength', 'subjectDistance',
                 'isoSpeed', 'exposureMode', 'meteringMode', 'macroMode',
                 'imageQuality', 'whiteBalance', 'orientation', 'sceneMode',
                 'sceneCaptureType', 'lensName', 'saturation', 'sharpness',
                 'contrast', 'fNumber', 'serialNumber', 'afPoint',
                 'shutterSpeedValue', 'brightnessValue', 'maxApertureValue',
                 'lightSource', 'subjectArea', 'sensingMethod'):
        if hasattr(exiv2, name):
            datum = getattr(exiv2, name)(exifData)
            if datum:
                print('{:18s}: {:30s}: {:s}'.format(
                    name, datum.key(), datum._print()))
            else:
                print(name)

    return 0


if __name__ == "__main__":
    sys.exit(main())



# python-exiv2 - Python interface to libexiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-23  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, Extension
import os
import subprocess
import sys


def pkg_config(library, option):
    cmd = ['pkg-config', '--' + option, library]
    try:
        return subprocess.check_output(cmd, universal_newlines=True).strip()
    except Exception as ex:
        print(str(ex))
        return None

def get_version(inc_dir):
    with open(os.path.join(inc_dir, 'exiv2', 'exv_conf.h')) as cnf:
        for line in cnf.readlines():
            words = line.split()
            if len(words) < 3:
                continue
            if words[0] == '#define' and words[1] == 'EXV_PACKAGE_VERSION':
                return [int(x) for x in eval(words[2]).split('.')]
    return [0, 0]

# get list of available swigged versions
swigged_versions = []
for name in os.listdir('src'):
    parts = name.split('_')
    if parts[0] == 'swig' and parts[1] not in swigged_versions:
        swigged_versions.append([int(x) for x in parts[1].split('.')])
swigged_versions.sort(reverse=True)

def get_mod_src_dir(exiv2_version):
    for version in swigged_versions:
        if exiv2_version + [0] >= version:
            return os.path.join('src', 'swig_{}.{}.{}'.format(*version))
    return None

mod_src_dir = None
platform = sys.platform
if platform == 'win32' and 'GCC' in sys.version:
    platform = 'mingw'

packages = ['exiv2', 'exiv2.examples']
package_dir = {'exiv2.examples': 'examples'}
package_data = {'exiv2.examples': ['*.py', '*.rst']}

if 'EXIV2_ROOT' in os.environ:
    # use local copy of libexiv2
    packages.append('exiv2.lib')
    package_dir['exiv2.lib'] = None
    include_dirs = []
    library_dirs = []
    for root, dirs, files in os.walk(os.path.normpath(os.environ['EXIV2_ROOT'])):
        for file in files:
            if file == 'exiv2.hpp':
                include_dirs = [os.path.dirname(root)]
                break
            if file == 'exiv2.mo':
                if 'exiv2.messages' not in packages:
                    # add exiv2.messages package for libexiv2 localisation files
                    packages.append('exiv2.messages')
                    package_dir['exiv2.messages'] = os.path.dirname(
                        os.path.dirname(root))
                    package_data['exiv2.messages'] = ['*/LC_MESSAGES/exiv2.mo']
                break
            if file in ('exiv2.lib', 'libexiv2.dll.a'):
                # win32, mingw, cygwin
                library_dirs = [root]
                break
            parts = file.split('.')
            if not (parts[0] in ('exiv2', 'libexiv2')
                    or parts[0].startswith('cygexiv2')):
                continue
            if len(parts) == 2 and parts[1] == 'dll':
                # win32, mingw, cygwin
                package_dir['exiv2.lib'] = root
                package_data['exiv2.lib'] = [file]
                break
            if len(parts) == 3 and (parts[1] == 'so' or parts[2] == 'dylib'):
                # linux, darwin
                library_dirs = [root]
                package_dir['exiv2.lib'] = root
                package_data['exiv2.lib'] = [file]
                break
        if (include_dirs and library_dirs and package_dir['exiv2.lib']
                and 'exiv2.messages' in packages):
            break
    if not (include_dirs and library_dirs and package_dir['exiv2.lib']):
        print('ERROR: Include and library files not found')
        sys.exit(1)
    # get exiv2 version from include files
    exiv2_version = get_version(include_dirs[0])
    mod_src_dir = get_mod_src_dir(exiv2_version)
    if platform == 'linux':
        extra_link_args = ['-Wl,-rpath,$ORIGIN/lib']
    elif platform == 'darwin':
        extra_link_args = ['-Wl,-rpath,@loader_path/lib']
    else:
        extra_link_args = []
else:
    # use installed libexiv2
    exiv2_version = pkg_config('exiv2', 'modversion')
    if exiv2_version:
        exiv2_version = [int(x) for x in exiv2_version.split('.')]
        mod_src_dir = get_mod_src_dir(exiv2_version)
        if mod_src_dir:
            library_dirs = pkg_config('exiv2', 'libs-only-L').split('-L')
            library_dirs = [x.strip() for x in library_dirs]
            library_dirs = [x.replace(r'\ ', ' ') for x in library_dirs if x]
            include_dirs = pkg_config('exiv2', 'cflags-only-I').split('-I')
            include_dirs = [x.strip() for x in include_dirs]
            include_dirs = [x.replace(r'\ ', ' ') for x in include_dirs if x]
            extra_link_args = []

if not mod_src_dir:
    print('ERROR: No SWIG source for libexiv2 version {}'.format(exiv2_version))
    sys.exit(1)

print('Using libexiv2 v{} with SWIG files from {}'.format(
    '.'.join(map(str, exiv2_version)), mod_src_dir))

package_dir['exiv2'] = mod_src_dir

# create extension modules list
ext_modules = []
extra_compile_args = []
define_macros = [('PY_SSIZE_T_CLEAN', None),
                 ('SWIG_TYPE_TABLE', 'exiv2')]
if platform in ('linux', 'darwin', 'mingw'):
    extra_compile_args = [
        '-O3', '-Wno-unused-variable', '-Wno-unused-function',
        '-Wno-deprecated-declarations', '-Wno-deprecated']
    if platform in ['linux', 'mingw']:
        extra_compile_args.append('-Wno-unused-but-set-variable')
    if 'PYTHON_EXIV2_STRICT' in os.environ:
        extra_compile_args.append('-Werror')
    if exiv2_version >= [0, 28]:
        extra_compile_args.append('-std=gnu++17')
    else:
        extra_compile_args.append('-std=c++98')
if platform == 'win32':
    extra_compile_args = ['/wd4101', '/wd4290']
for file_name in os.listdir(mod_src_dir):
    if file_name[-9:] != '_wrap.cxx':
        continue
    ext_name = file_name[:-9]
    ext_modules.append(Extension(
        '_' + ext_name,
        sources = [os.path.join(mod_src_dir, file_name)],
        include_dirs = include_dirs,
        extra_compile_args = extra_compile_args,
        define_macros = define_macros,
        libraries = ['exiv2'],
        library_dirs = library_dirs,
        extra_link_args = extra_link_args,
        ))

# set options for building source distributions
command_options = {}
command_options['sdist'] = {
    'formats' : ('setup.py', 'zip'),
    }

with open('README.rst') as ldf:
    long_description = ldf.read()
py_exiv2_version = long_description.splitlines()[0].split()[-1]

setup(name = 'exiv2',
      version = py_exiv2_version,
      description = 'Python interface to libexiv2',
      long_description = long_description,
      author = 'Jim Easterbrook',
      author_email = 'jim@jim-easterbrook.me.uk',
      url = 'https://github.com/jim-easterbrook/python-exiv2',
      classifiers = [
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
          'Operating System :: MacOS',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: POSIX',
          'Operating System :: POSIX :: Linux',
          'Operating System :: Microsoft',
          'Operating System :: Microsoft :: Windows',
          'Programming Language :: Python :: 3',
          'Topic :: Multimedia',
          'Topic :: Multimedia :: Graphics',
          ],
      platforms = ['POSIX', 'MacOS', 'Windows'],
      license = 'GNU GPL',
      command_options = command_options,
      ext_package = 'exiv2',
      ext_modules = ext_modules,
      packages = packages,
      package_dir = package_dir,
      package_data = package_data,
      zip_safe = False,
      )



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2022-23  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import unittest

import exiv2


class TestDataRead(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        exiv2.XmpParser.initialize()
        test_dir = os.path.dirname(__file__)
        image = exiv2.ImageFactory.open(os.path.join(test_dir, 'image_02.jpg'))
        image.readMetadata()
        cls.exifData = image.exifData()
        cls.iptcData = image.iptcData()
        cls.xmpData = image.xmpData()

    def test_exif(self):
        for tag, exiv_type, value in (
                ('Exif.Image.Copyright', exiv2.AsciiValue,
                 'Copyright ©2022 Jim Easterbrook. All rights reserved.'),
                ('Exif.Image.Orientation', exiv2.UShortValue, [1]),
                ('Exif.Photo.LensSpecification', exiv2.URationalValue,
                 [(18, 1), (200, 1), (7, 2), (63, 10)]),
                ('Exif.Thumbnail.ImageWidth', exiv2.ULongValue, [160]),
                ):
            datum = self.exifData[tag]
            exiv_value = datum.value()
            self.assertIsInstance(exiv_value, exiv_type)
            if exiv_type == exiv2.AsciiValue:
                self.assertEqual(str(exiv_value), value)
                self.assertEqual(
                    bytes(exiv_value.data()), value.encode('utf-8'))
            else:
                self.assertEqual(len(exiv_value), len(value))
                self.assertEqual(list(exiv_value), value)
        thumb = exiv2.ExifThumb(self.exifData).copy()
        data = memoryview(thumb.data())
        self.assertEqual(len(data), 2532)
        self.assertEqual(
            data[:15], b'\xff\xd8\xff\xe0\x00\x10JFIF\x00\x01\x01\x00\x00')

    def test_iptc(self):
        datum = self.iptcData['Iptc.Application2.Caption']
        exiv_value = datum.value()
        self.assertIsInstance(exiv_value, exiv2.StringValue)
        self.assertEqual(str(exiv_value), 'Good view of the lighthouse.')

        datum = self.iptcData['Iptc.Application2.DateCreated']
        exiv_value = datum.value()
        self.assertIsInstance(exiv_value, exiv2.DateValue)
        self.assertEqual(dict(exiv_value.getDate()),
                         {'year': 2022, 'month': 8, 'day': 17})

        datum = self.iptcData['Iptc.Application2.TimeCreated']
        exiv_value = datum.value()
        self.assertIsInstance(exiv_value, exiv2.TimeValue)
        self.assertEqual(dict(exiv_value.getTime()),
                         {'hour': 12, 'minute': 45, 'second': 28,
                          'tzHour': 1, 'tzMinute': 0})

    def test_xmp(self):
        for tag, exiv_type, value in (
                ('Xmp.photoshop.Credit', exiv2.XmpTextValue,
                 'Jim Easterbrook'),
                ('Xmp.dc.creator', exiv2.XmpArrayValue, ['Jim Easterbrook']),
                ('Xmp.dc.subject', exiv2.XmpArrayValue,
                 ['lighthouse', 'Scotland']),
                ('Xmp.dc.description', exiv2.LangAltValue,
                 {'x-default': 'Good view of the lighthouse.',
                  'en-GB': 'Good view of the lighthouse.',
                  'de': 'Gute Sicht auf den Leuchtturm.'}),
                ):
            datum = self.xmpData[tag]
            exiv_value = datum.value()
            self.assertIsInstance(exiv_value, exiv_type)
            if exiv_type == exiv2.XmpArrayValue:
                self.assertEqual(list(exiv_value), value)
            elif exiv_type == exiv2.LangAltValue:
                self.assertEqual(dict(exiv_value), value)
            else:
                self.assertEqual(str(exiv_value), value)

    def test_xmp_complex(self):
        for tag, struct, array in (
                ('Xmp.iptcExt.LocationCreated',
                 exiv2.XmpStruct.xsNone, exiv2.XmpArrayType.xaBag),
                ('Xmp.iptcExt.LocationCreated[1]',
                 exiv2.XmpStruct.xsStruct, exiv2.XmpArrayType.xaNone)):
            datum = self.xmpData[tag]
            value = datum.value()
            self.assertEqual(value.xmpStruct(), struct)
            self.assertEqual(value.xmpArrayType(), array)

    def test_set_value(self):
        datum = self.exifData['Exif.GPSInfo.GPSLatitude']
        value = datum.getValue()
        # set value
        self.assertEqual(list(value), [(57, 1), (51, 1), (146751, 5000)])
        value[1] = (45, 1)
        self.assertEqual(list(value), [(57, 1), (45, 1), (146751, 5000)])
        del value[1]
        self.assertEqual(list(value), [(57, 1), (146751, 5000)])
        value.append((12, 34))
        self.assertEqual(list(value), [(57, 1), (146751, 5000), (12, 34)])
        # set datum
        self.assertEqual(list(datum.value()), [(57, 1), (51, 1), (146751, 5000)])
        self.exifData['Exif.GPSInfo.GPSLatitude'] = exiv2.URationalValue(
            [(23, 1), (47, 1), (3592, 100)])
        self.assertEqual(list(datum.value()), [(23, 1), (47, 1), (3592, 100)])
        self.exifData['Exif.GPSInfo.GPSLatitude'] = '23/1 46/1'
        self.assertEqual(list(datum.value()), [(23, 1), (46, 1)])
        del self.exifData['Exif.GPSInfo.GPSLatitude']
        datum = self.exifData['Exif.GPSInfo.GPSLatitude']
        with self.assertRaises(exiv2.Exiv2Error):
            self.assertEqual(datum.value(), [])


if __name__ == '__main__':
    unittest.main()



##  python-exiv2 - Python interface to libexiv2
##  http://github.com/jim-easterbrook/python-exiv2
##  Copyright (C) 2022  Jim Easterbrook  jim@jim-easterbrook.me.uk
##
##  This program is free software: you can redistribute it and/or
##  modify it under the terms of the GNU General Public License as
##  published by the Free Software Foundation, either version 3 of the
##  License, or (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see
##  <http://www.gnu.org/licenses/>.

import os
import unittest

import exiv2


class TestIterators(unittest.TestCase):
    def setUp(self):
        exiv2.XmpParser.initialize()
        test_dir = os.path.dirname(__file__)
        image = exiv2.ImageFactory.open(os.path.join(test_dir, 'image_01.jpg'))
        image.readMetadata()
        self.exifData = image.exifData()
        self.iptcData = image.iptcData()
        self.xmpData = image.xmpData()

    def iterator_test_python_style(self, data, result):
        keys = []
        for datum in data:
            keys.append(datum.key())
        self.assertEqual(keys, result)

    def iterator_test_c_style(self, data, result):
        keys = []
        b = data.begin()
        e = data.end()
        while b != e:
            keys.append(b.key())
            next(b)
        self.assertEqual(keys, result)

    def iterator_test_mixed(self, data, result):
        keys = []
        for datum in data.begin():
            keys.append(datum.key())
        self.assertEqual(keys, result)

    def test_iterators(self):
        exif_keys = [
            'Exif.Image.ProcessingSoftware', 'Exif.Image.ImageDescription',
            'Exif.Image.DateTime', 'Exif.Image.ExifTag',
            'Exif.Photo.DateTimeOriginal', 'Exif.Photo.DateTimeDigitized',
            'Exif.Photo.SubSecTime', 'Exif.Photo.SubSecTimeOriginal',
            'Exif.Photo.SubSecTimeDigitized']
        self.iterator_test_python_style(self.exifData, exif_keys)
        self.iterator_test_c_style(self.exifData, exif_keys)
        self.iterator_test_mixed(self.exifData, exif_keys)
        iptc_keys = [
            'Iptc.Envelope.CharacterSet', 'Iptc.Application2.DigitizationDate',
            'Iptc.Application2.DigitizationTime',
            'Iptc.Application2.DateCreated', 'Iptc.Application2.TimeCreated',
            'Iptc.Application2.Caption', 'Iptc.Application2.Keywords',
            'Iptc.Application2.Keywords', 'Iptc.Application2.Program',
            'Iptc.Application2.ProgramVersion']
        self.iterator_test_python_style(self.iptcData, iptc_keys)
        self.iterator_test_c_style(self.iptcData, iptc_keys)
        self.iterator_test_mixed(self.iptcData, iptc_keys)
        xmp_keys = [
            'Xmp.xmp.CreateDate', 'Xmp.xmp.ModifyDate',
            'Xmp.photoshop.DateCreated', 'Xmp.dc.description',
            'Xmp.dc.subject']
        self.iterator_test_python_style(self.xmpData, xmp_keys)
        self.iterator_test_c_style(self.xmpData, xmp_keys)
        self.iterator_test_mixed(self.xmpData, xmp_keys)

    def find_erase_test(self, data, key, value):
        pos = data.findKey(key)
        self.assertNotEqual(pos, data.end())
        self.assertEqual(pos.key(), key.key())
        self.assertEqual(pos.toString(), value)
        data.erase(pos)
        pos = data.findKey(key)
        self.assertEqual(pos, data.end())

    def test_find_erase(self):
        self.find_erase_test(
            self.exifData, exiv2.ExifKey('Exif.Image.ImageDescription'),
            'Description')
        self.find_erase_test(
            self.iptcData, exiv2.IptcKey('Iptc.Application2.Caption'),
            'Description')
        self.find_erase_test(
            self.xmpData, exiv2.XmpKey('Xmp.dc.description'),
            'lang="en-GB" Description')

    def test_empty(self):
        data = exiv2.ExifData()
        self.assertEqual(data.begin(), data.end())
        for datum in data:
            self.fail('not empty')
        for datum in data.begin():
            self.fail('not empty')
        for datum in data.end():
            self.fail('not empty')


if __name__ == '__main__':
    unittest.main()



# python-exiv2 - Python interface to exiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import exiv2

def main():
    print('libexiv2 version:', exiv2.version())
    print('python-exiv2 version:', exiv2.__version__)
    print('python-exiv2 examples:',
          os.path.join(os.path.dirname(__file__), 'examples'))
    if exiv2.version() >= '0.27.4':
        print('BMFF support:', exiv2.enableBMFF(False))

if __name__ == "__main__":
    sys.exit(main())



#  Photini - a simple photo metadata editor.
#  http://github.com/jim-easterbrook/Photini
#  Copyright (C) 2020  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
#  This program is free software: you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see
#  <http://www.gnu.org/licenses/>.

from datetime import date
import re
import sys

# requires GitPython - 'sudo pip install gitpython'
import git


def main(argv=None):
    with open('README.rst') as rst:
        py_exiv2_version = rst.readline().split()[-1]
    message = 'python-exiv2-' + py_exiv2_version + '\n\n'
    with open('CHANGELOG.txt') as cl:
        while not cl.readline().startswith('Changes'):
            pass
        while True:
            line = cl.readline().strip()
            if not line:
                break
            message += line + '\n'
    repo = git.Repo()
    tag = repo.create_tag(py_exiv2_version, message=message)
    remote = repo.remotes.origin
    remote.push(tags=True)
    return 0


if __name__ == "__main__":
    sys.exit(main())



# python-exiv2 - Python interface to exiv2
# http://github.com/jim-easterbrook/python-exiv2
# Copyright (C) 2021-22  Jim Easterbrook  jim@jim-easterbrook.me.uk
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import shutil
import subprocess
import sys
import tempfile


def get_version(incl_dir):
    with open(os.path.join(incl_dir, 'exv_conf.h')) as cnf:
        for line in cnf.readlines():
            words = line.split()
            if len(words) < 3:
                continue
            if words[0] == '#define' and words[1] == 'EXV_PACKAGE_VERSION':
                version = [int(x) for x in eval(words[2]).split('.')]
                return tuple(version + [0, 0])
    return 0, 0, 0, 0


def main():
    # get version to SWIG
    if len(sys.argv) < 2 or len(sys.argv) > 3:
        print('Usage: %s path ["minimal"]' % sys.argv[0])
        return 1
    # minimal build?
    minimal = len(sys.argv) >= 3 and sys.argv[2] == 'minimal'
    # get config
    platform = sys.platform
    if platform == 'win32' and 'GCC' in sys.version:
        platform = 'mingw'
    incl_dir = os.path.normpath(sys.argv[1])
    if os.path.basename(incl_dir) != 'exiv2':
        incl_dir = os.path.join(incl_dir, 'exiv2')
    if not os.path.isdir(incl_dir):
        print('Directory %s not found' % incl_dir)
        return 2
    if 'exiv2.hpp' not in os.listdir(incl_dir):
        print('Exiv2 header files not found in %s' % incl_dir)
        return 3
    # get exiv2 version
    exiv2_version = get_version(incl_dir)
    # get exiv2 build options
    options = {
        'EXV_ENABLE_BMFF'  : False,
        'EXV_ENABLE_NLS'   : False,
        'EXV_ENABLE_VIDEO' : False,
        'EXV_UNICODE_PATH' : False,
        }
    if not minimal:
        with open(os.path.join(incl_dir, 'exv_conf.h')) as cnf:
            for line in cnf.readlines():
                words = line.split()
                for key in options:
                    if key not in line:
                        continue
                    if words[1] != key:
                        continue
                    if words[0] == '#define':
                        options[key] = True
                    elif words[0] == '#undef':
                        options[key] = False
    # get python-exiv2 version
    with open('README.rst') as rst:
        py_exiv2_version = rst.readline().split()[-1]
    # get list of modules (Python) and extensions (SWIG)
    interface_dir = os.path.join('src', 'interface')
    file_names = os.listdir(interface_dir)
    file_names = [x for x in file_names if x != 'preamble.i']
    file_names.sort()
    file_names = [os.path.splitext(x) for x in file_names]
    mod_names = [x[0] + x[1] for x in file_names if x[1] == '.py']
    ext_names = [x[0] for x in file_names if x[1] == '.i']
    # get SWIG version
    cmd = ['swig', '-version']
    try:
        swig_version = str(subprocess.Popen(
            cmd, stdout=subprocess.PIPE,
            universal_newlines=True).communicate()[0])
    except Exception:
        print('ERROR: command "%s" failed' % ' '.join(cmd))
        raise
    for line in swig_version.splitlines():
        if 'Version' in line:
            swig_version = tuple(map(int, line.split()[-1].split('.')))
            break
    # convert exiv2 version to hex
    exiv2_version_hex = '0x{:02x}{:02x}{:02x}{:02x}'.format(*exiv2_version)
    # create output dir
    output_dir = os.path.join('src', 'swig_{}.{}.{}'.format(*exiv2_version))
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    # copy Python modules
    for mod_name in mod_names:
        shutil.copy2(os.path.join('src', 'interface', mod_name),
                     os.path.join(output_dir, mod_name))
    # pre-process include files to a temporary directory
    with tempfile.TemporaryDirectory() as copy_dir:
        dest = os.path.join(copy_dir, 'exiv2')
        os.makedirs(dest)
        attr = re.compile(r'^\s*?(\[\[.*?\]\]\s*?)')
        for file in os.listdir(incl_dir):
            with open(os.path.join(incl_dir, file), 'r') as in_file:
                with open(os.path.join(dest, file), 'w') as out_file:
                    for line in in_file.readlines():
                        if 'static constexpr auto' in line:
                            continue
                        line = attr.sub('', line)
                        out_file.write(line)
        # make options list
        swig_opts = ['-c++', '-python', '-builtin',
                     '-fastdispatch', '-fastproxy',
                     '-Wextra', '-Werror']
        if swig_version < (4, 1, 0):
            swig_opts.append('-py3')
        swig_opts.append('-I' + copy_dir)
        for key in options:
            if options[key]:
                swig_opts.append('-D' + key)
        swig_opts.append('-DEXIV2_VERSION_HEX=' + exiv2_version_hex)
        swig_opts += ['-outdir', output_dir]
        # do each swig module
        for ext_name in ext_names:
            cmd = ['swig'] + swig_opts
            # use -doxygen ?
            if swig_version >= (4, 0, 0):
                # -doxygen flag causes a syntax error on error.hpp in v0.26
                if exiv2_version >= (0, 27) or ext_name not in ('error', ):
                    cmd += ['-doxygen', '-DSWIG_DOXYGEN']
            cmd += ['-o', os.path.join(output_dir, ext_name + '_wrap.cxx')]
            cmd += [os.path.join(interface_dir, ext_name + '.i')]
            print(' '.join(cmd))
            subprocess.check_call(cmd)
    # create init module
    init_file = os.path.join(output_dir, '__init__.py')
    with open(init_file, 'w') as im:
        im.write('''
import logging
import os
import sys
import warnings

if sys.platform == 'win32':
    _dir = os.path.join(os.path.dirname(__file__), 'lib')
    if os.path.isdir(_dir):
        if hasattr(os, 'add_dll_directory'):
            os.add_dll_directory(_dir)
        os.environ['PATH'] = _dir + ';' + os.environ['PATH']

_logger = logging.getLogger(__name__)

class Exiv2Error(Exception):
    """Python exception raised by exiv2 library errors"""
    pass

if sys.version_info < (3, 7):
    # provide old AnyError for compatibility
    AnyError = Exiv2Error
else:
    # issue deprecation warning if user imports AnyError
    def __getattr__(name):
        if name == 'AnyError':
            warnings.warn("Please replace 'AnyError' with 'Exiv2Error'",
                          DeprecationWarning)
            return Exiv2Error
        raise AttributeError

''')
        im.write('__version__ = "%s"\n\n' % py_exiv2_version)
        for name in ext_names:
            im.write('from exiv2.%s import *\n' % name)
        im.write('''
_dir = os.path.join(os.path.dirname(__file__), 'messages')
if os.path.isdir(_dir):
    exiv2.types._set_locale_dir(_dir)

__all__ = [x for x in dir() if x[0] != '_']
''')
    return 0


if __name__ == "__main__":
    sys.exit(main())
